@@ -69,7 +69,6 @@ use Carp qw(longmess);
 use File::Temp;
 use Data::Dumper;
 use Siebel::Srvrmgr;
-use File::BOM qw(:all);
 
 extends 'Siebel::Srvrmgr::Daemon';
 
@@ -153,16 +152,9 @@ sub run {
 
     $self->_check_system( $logger, ${^CHILD_ERROR_NATIVE}, $ret_code, $? );
 
-    my $in;
-
-    eval { open_bom( $in, $self->get_output_file(),':utf8' ) };
-
-    if ($@) {
-
-      $logger->logdie( 'Cannot read ' . $self->get_output_file() . ': ' . $@ );
-
-    }
-
+    open( my $in, '<', $self->get_output_file() )
+      or
+      $logger->logdie( 'Cannot read ' . $self->get_output_file() . ': ' . $! );
     my @input_buffer = <$in>;
     close($in);
 
@@ -171,7 +163,7 @@ sub run {
         $self->normalize_eol( \@input_buffer );
         chomp(@input_buffer);
 
-# since we should have all output, we parse everything first to call each action after
+# since we should have all output, we parse everthing first to call each action after
         $parser->parse( \@input_buffer );
 
         if ( $parser->has_tree() ) {
