@@ -11,6 +11,7 @@ Siebel::Srvrmgr::ListParser::OutputFactory - abstract factory class to create Si
 use warnings;
 use strict;
 use MooseX::AbstractFactory;
+use Carp;
 
 =pod
 
@@ -27,6 +28,8 @@ use MooseX::AbstractFactory;
         }
     );
 
+	if (Siebel::Srvrmgr::ListParser::OutputFactory->can_create('weirdo')) ? print "can\n" : print "cannot\n";
+
 =head1 DESCRIPTION
 
 This is an abstract factory class to create instances of subclass of L<Siebel::Srvrmgr::ListParser::Output> superclass.
@@ -68,6 +71,7 @@ L<Siebel::Srvrmgr::ListParser>
 
 =cut
 
+ # :TODO      :01/07/2013 13:37:06:: create "static" method to return this data
 our %table_mapping = (
     'list_comp'        => 'ListComp',
     'list_params'      => 'ListParams',
@@ -101,7 +105,7 @@ implementation_class_via sub {
     }
     else {
 
-        die "Cannot defined a class for command $last_cmd_type";
+        confess "Cannot defined a class for command '$last_cmd_type'";
 
     }
 
@@ -111,11 +115,11 @@ implementation_class_via sub {
 
 =head1 AUTHOR
 
-Alceu Rodrigues de Freitas Junior, E<lt>arfreitas@cpan.org<E<gt>
+Alceu Rodrigues de Freitas Junior, E<lt>arfreitas@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2012 of Alceu Rodrigues de Freitas Junior, E<lt>arfreitas@cpan.org<E<gt>
+This software is copyright (c) 2012 of Alceu Rodrigues de Freitas Junior, E<lt>arfreitas@cpan.orgE<gt>.
 
 This file is part of Siebel Monitoring Tools.
 
@@ -130,7 +134,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with Siebel Monitoring Tools.  If not, see <http://www.gnu.org/licenses/>.
+along with Siebel Monitoring Tools.  If not, see L<http://www.gnu.org/licenses/>.
 
 =cut
 
